#include "SmartRFID.hpp"

SmartRFID::SmartRFID(Adafruit_PN532 *nfc)
{
    m_nfc = nfc; // RFID with I2C
}

void SmartRFID::init()
{
    m_nfc->begin();
    unsigned long versiondata = m_nfc->getFirmwareVersion();
    if (!versiondata)
    {
        Serial.print("No board found!");
        while (1)
            ;
    }
    Serial.println("PN5 chip found!");
}

String SmartRFID::readTag(uint16_t timeoutMillis)
{
    m_nfc->SAMConfig();

    uint8_t success;
    uint8_t uid[] = {0, 0, 0, 0, 0, 0, 0}; // buffer for UID
    uint8_t uidLength;                     // UID length (4 or 7 bytes depending on ISO14443A Card/Chip type)
    success = m_nfc->readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength, timeoutMillis);
    if (success)
    {
        Serial.println("Found an ISO14443A card");
        Serial.print("  UID Length: ");
        Serial.print(uidLength, DEC);
        Serial.println(" bytes");
        Serial.print("  UID Value: ");
        m_nfc->PrintHex(uid, uidLength);
        Serial.println("");
        Serial.print("  compact hex code: ");
        String s = string_dump_byte_array(uid, uidLength);
        Serial.println(s);
        return s;
    }
    return "no_tag_detected";
}

/**
 * Helper routine to dump a byte array as hex values to a string.
 */
String SmartRFID::string_dump_byte_array(byte *buffer, byte bufferSize)
{
    String result = "";
    for (byte i = 0; i < bufferSize; i++)
    {
        result += (buffer[i] < 0x10 ? " 0" : " ");
        result += String(buffer[i], HEX);
    }
    result.replace(" ", "");
    return result;
}